==> koffice2_1.9.93.orig.tar.gz: koffice-1.9.93/krita/plugins/tools/selectiontools/kis_tool_select_path.cc <==
/*
 *  Copyright (c) 2007 Sven Langkamp <sven.langkamp@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <QApplication>
#include <QPainter>
#include <QPen>
#include <QLayout>
#include <QVBoxLayout>

#include <klocale.h>

#include <KoShapeController.h>
#include <KoPathShape.h>
#include <KoShapeManager.h>
#include <KoShapeRegistry.h>
#include <KoPointerEvent.h>

#include "kis_image.h"
#include "kis_painter.h"
#include "kis_layer.h"
#include "kis_tool_select_path.h"
#include "kis_selection.h"
#include "kis_selection_options.h"
#include "kis_selected_transaction.h"
#include "kis_canvas2.h"
#include "kis_shape_selection.h"
#include "kis_pixel_selection.h"
#include "kis_resource_provider.h"
#include "kis_paintop_registry.h"


KisToolSelectPath::KisToolSelectPath(KoCanvasBase * canvas)
    : KoCreatePathTool(canvas)
{
    m_optWidget = 0;
    m_selectAction = SELECTION_REPLACE;
    m_selectionMode = PIXEL_SELECTION;
}

KisToolSelectPath::~KisToolSelectPath()
{
}

void KisToolSelectPath::activate()
{
    super::activate();

    if (!m_optWidget)
        return;

    m_optWidget->slotActivated();
}

void KisToolSelectPath::slotSetAction(int action) {
    if (action >= SELECTION_ADD && action <= SELECTION_INTERSECT)
        m_selectAction =(selectionAction)action;
}

void KisToolSelectPath::slotSetSelectionMode(int mode) {
    m_selectionMode = (selectionMode)mode;

}

QWidget* KisToolSelectPath::createOptionWidget()
{
    KisCanvas2* canvas = dynamic_cast<KisCanvas2*>(m_canvas);
    Q_ASSERT(canvas);
    m_optWidget = new KisSelectionOptions(canvas);
    Q_CHECK_PTR(m_optWidget);
    m_optWidget->setWindowTitle(i18n("Path Selection"));
    m_optWidget->disableAntiAliasSelectionOption();

    connect(m_optWidget, SIGNAL(actionChanged(int)), this, SLOT(slotSetAction(int)));
    connect(m_optWidget, SIGNAL(modeChanged(int)), this, SLOT(slotSetSelectionMode(int)));


    QVBoxLayout * l = dynamic_cast<QVBoxLayout*>(m_optWidget->layout());
    Q_ASSERT(l);
    if (l) {
        l->addItem(new QSpacerItem(1, 1, QSizePolicy::Fixed, QSizePolicy::Expanding));
    }

    return m_optWidget;
}

QWidget* KisToolSelectPath::optionWidget()
{
        return m_optWidget;
}

void KisToolSelectPath::addPathShape()
{
    KisLayerSP currentLayer = m_canvas->resourceProvider()->resource( KisResourceProvider::CurrentKritaLayer ).value<KisLayerSP>();
    if(!currentLayer)
        return;

    KisImageSP image = currentLayer->image();
    KisPaintDeviceSP dev = currentLayer->paintDevice();
    bool hasSelection = dev->hasSelection();

    m_shape->normalize();

    KoPathShape *shape = m_shape;
    shape->close();
    m_shape = 0;

    if(m_selectionMode == PIXEL_SELECTION){
        KisSelectedTransaction *t = new KisSelectedTransaction(i18n("Path Selection"), dev);
        KisPixelSelectionSP getOrCreatePixelSelection = dev->selection()->getOrCreatePixelSelection();


        if(! hasSelection || m_selectAction == SELECTION_REPLACE)
        {
            getOrCreatePixelSelection->clear();
            if(m_selectAction==SELECTION_SUBTRACT)
                getOrCreatePixelSelection->invert();
        }

        KisPixelSelectionSP tmpSel = KisPixelSelectionSP(new KisPixelSelection(dev));

        KisPainter painter(tmpSel);
        painter.setPaintColor(KoColor(Qt::black, tmpSel->colorSpace()));
        painter.setFillStyle(KisPainter::FillStyleForegroundColor);
        painter.setStrokeStyle(KisPainter::StrokeStyleNone);
        painter.setOpacity(OPACITY_OPAQUE);
        painter.setCompositeOp(tmpSel->colorSpace()->compositeOp(COMPOSITE_OVER));
        KisPaintOp * op = KisPaintOpRegistry::instance()->paintOp("paintbrush", 0, &painter, image);
        painter.setPaintOp(op);

        QMatrix matrix;
        matrix.scale(image->xRes(), image->yRes());
        matrix.translate(shape->position().x(), shape->position().y());
        painter.fillPainterPath(matrix.map(shape->outline()));

        switch(m_selectAction)
        {
            case SELECTION_REPLACE:
            case SELECTION_ADD:
                getOrCreatePixelSelection->addSelection(tmpSel);
                break;
            case SELECTION_SUBTRACT:
                getOrCreatePixelSelection->subtractSelection(tmpSel);
                break;
            case SELECTION_INTERSECT:
                getOrCreatePixelSelection->intersectSelection(tmpSel);
                break;
            default:
                break;
        }
        if(hasSelection && m_selectAction != SELECTION_REPLACE && m_selectAction != SELECTION_INTERSECT) {
            QRect rect(painter.dirtyRegion().boundingRect());
            dev->setDirty(rect);
            dev->emitSelectionChanged(rect);
        } else {
            dev->setDirty(image->bounds());
            dev->emitSelectionChanged();
        }
        m_canvas->addCommand(t);
        delete shape;
    }
    else {
        KisSelectionSP selection = dev->selection();

        KisShapeSelection* shapeSelection;
        if(!selection->hasShapeSelection()) {
            shapeSelection = new KisShapeSelection(image, dev);
            QUndoCommand * cmd = m_canvas->shapeController()->addShape(shapeSelection);
            cmd->redo();
            selection->setShapeSelection(shapeSelection);
        }
        else {
            shapeSelection = static_cast<KisShapeSelection*>(selection->shapeSelection());
        }
        QUndoCommand * cmd = m_canvas->shapeController()->addShape(shape);
        m_canvas->addCommand(cmd);
        shapeSelection->addChild(shape);
    }
}

#include "kis_tool_select_path.moc"

==> koffice2_1.9.95.orig.tar.gz: koffice-1.9.95/krita/plugins/tools/selectiontools/kis_tool_select_path.cc <==
/*
 *  Copyright (c) 2007 Sven Langkamp <sven.langkamp@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <QApplication>
#include <QPainter>
#include <QPen>
#include <QLayout>
#include <QVBoxLayout>

#include <klocale.h>

#include <KoShapeController.h>
#include <KoPathShape.h>
#include <KoShapeManager.h>
#include <KoShapeRegistry.h>
#include <KoPointerEvent.h>

#include "kis_image.h"
#include "kis_painter.h"
#include "kis_layer.h"
#include "kis_tool_select_path.h"
#include "kis_selection.h"
#include "kis_selection_options.h"
#include "kis_canvas2.h"
#include "kis_selection_tool_helper.h"
#include "kis_pixel_selection.h"
#include "kis_resource_provider.h"
#include "kis_paintop_registry.h"


KisToolSelectPath::KisToolSelectPath(KoCanvasBase * canvas)
    : KoCreatePathTool(canvas)
{
    m_optWidget = 0;
    m_selectAction = SELECTION_REPLACE;
    m_selectionMode = PIXEL_SELECTION;
}

KisToolSelectPath::~KisToolSelectPath()
{
}

void KisToolSelectPath::activate()
{
    super::activate();

    if (!m_optWidget)
        return;

    m_optWidget->slotActivated();
}

void KisToolSelectPath::slotSetAction(int action) {
    if (action >= SELECTION_REPLACE && action <= SELECTION_INTERSECT)
        m_selectAction =(selectionAction)action;
}

void KisToolSelectPath::slotSetSelectionMode(int mode) {
    m_selectionMode = (selectionMode)mode;

}

QWidget* KisToolSelectPath::createOptionWidget()
{
    KisCanvas2* canvas = dynamic_cast<KisCanvas2*>(m_canvas);
    Q_ASSERT(canvas);
    m_optWidget = new KisSelectionOptions(canvas);
    Q_CHECK_PTR(m_optWidget);
    m_optWidget->setWindowTitle(i18n("Path Selection"));
    m_optWidget->disableAntiAliasSelectionOption();

    connect(m_optWidget, SIGNAL(actionChanged(int)), this, SLOT(slotSetAction(int)));
    connect(m_optWidget, SIGNAL(modeChanged(int)), this, SLOT(slotSetSelectionMode(int)));


    QVBoxLayout * l = dynamic_cast<QVBoxLayout*>(m_optWidget->layout());
    Q_ASSERT(l);
    if (l) {
        l->addItem(new QSpacerItem(1, 1, QSizePolicy::Fixed, QSizePolicy::Expanding));
    }

    return m_optWidget;
}

QWidget* KisToolSelectPath::optionWidget()
{
        return m_optWidget;
}

void KisToolSelectPath::addPathShape()
{
    KisLayerSP currentLayer = m_canvas->resourceProvider()->resource( KisResourceProvider::CurrentKritaLayer ).value<KisLayerSP>();
    if(!currentLayer)
        return;

    KisImageSP image = currentLayer->image();
    KisPaintDeviceSP dev = currentLayer->paintDevice();
    bool hasSelection = currentLayer->selection();

    m_shape->normalize();

    KoPathShape *shape = m_shape;
    shape->close();
    m_shape = 0;

    KisSelectionToolHelper helper(m_canvas->shapeController(), currentLayer, i18n("Path Selection"));

    if( m_selectionMode == PIXEL_SELECTION ){

        KisPixelSelectionSP tmpSel = KisPixelSelectionSP(new KisPixelSelection(dev));

        KisPainter painter(tmpSel);
        painter.setPaintColor(KoColor(Qt::black, tmpSel->colorSpace()));
        painter.setFillStyle(KisPainter::FillStyleForegroundColor);
        painter.setStrokeStyle(KisPainter::StrokeStyleNone);
        painter.setOpacity(OPACITY_OPAQUE);
        painter.setCompositeOp(tmpSel->colorSpace()->compositeOp(COMPOSITE_OVER));
        KisPaintOp * op = KisPaintOpRegistry::instance()->paintOp("paintbrush", 0, &painter, image);
        painter.setPaintOp(op);

        QMatrix matrix;
        matrix.scale(image->xRes(), image->yRes());
        matrix.translate(shape->position().x(), shape->position().y());
        painter.fillPainterPath(matrix.map(shape->outline()));

        QUndoCommand* cmd = helper.selectPixelSelection(tmpSel, m_selectAction);
        m_canvas->addCommand(cmd);

        delete shape;
    }
    else {
        QUndoCommand* cmd = helper.addSelectionShape(shape);
        m_canvas->addCommand(cmd);
    }
}

#include "kis_tool_select_path.moc"
